@@ -12,17 +12,17 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include "lib.h"
-#include "metadata.h"
-#include "locking.h"
-#include "lvm-string.h"
-#include "toolcontext.h"
-#include "display.h"
-#include "segtype.h"
-#include "activate.h"
-#include "defaults.h"
-#include "lv_alloc.h"
-#include "lvm-signal.h"
+#include "lib/misc/lib.h"
+#include "lib/metadata/metadata.h"
+#include "lib/locking/locking.h"
+#include "lib/misc/lvm-string.h"
+#include "lib/commands/toolcontext.h"
+#include "lib/display/display.h"
+#include "lib/metadata/segtype.h"
+#include "lib/activate/activate.h"
+#include "lib/config/defaults.h"
+#include "lib/metadata/lv_alloc.h"
+#include "lib/misc/lvm-signal.h"
 
 /* https://github.com/jthornber/thin-provisioning-tools/blob/master/caching/cache_metadata_size.cc */
 #define DM_TRANSACTION_OVERHEAD		4096  /* KiB */
@@ -131,6 +131,10 @@ void cache_check_for_warns(const struct lv_segment *seg)
 	    first_seg(seg->pool_lv)->cache_mode == CACHE_MODE_WRITEBACK)
 		log_warn("WARNING: Data redundancy could be lost with writeback "
 			 "caching of raid logical volume!");
+
+	if (lv_is_thin_pool_data(seg->lv))
+		log_warn("WARNING: Cached thin pool's data cannot be currently "
+			 "resized and require manual uncache before resize!");
 }
 
 /*
@@ -157,7 +161,6 @@ int update_cache_pool_params(struct cmd_context *cmd,
 			     unsigned attr,
 			     uint32_t pool_data_extents,
 			     uint32_t *pool_metadata_extents,
-			     struct logical_volume *metadata_lv,
 			     int *chunk_size_calc_method, uint32_t *chunk_size)
 {
 	uint64_t min_meta_size;
@@ -203,26 +206,39 @@ int update_cache_pool_params(struct cmd_context *cmd,
 	if (!validate_cache_chunk_size(cmd, *chunk_size))
 		return_0;
 
+	min_meta_size = _cache_min_metadata_size((uint64_t) pool_data_extents * extent_size, *chunk_size);
+
+	/* Round up to extent size */
+	if (min_meta_size % extent_size)
+		min_meta_size += extent_size - min_meta_size % extent_size;
+
+	if (!pool_metadata_size)
+		pool_metadata_size = min_meta_size;
+
+	if (pool_metadata_size > (2 * DEFAULT_CACHE_POOL_MAX_METADATA_SIZE)) {
+		pool_metadata_size = 2 * DEFAULT_CACHE_POOL_MAX_METADATA_SIZE;
+		if (*pool_metadata_extents)
+			log_warn("WARNING: Maximum supported pool metadata size is %s.",
+				 display_size(cmd, pool_metadata_size));
+	} else if (pool_metadata_size < min_meta_size) {
+		if (*pool_metadata_extents)
+			log_warn("WARNING: Minimum required pool metadata size is %s "
+				 "(needs extra %s).",
+				 display_size(cmd, min_meta_size),
+				 display_size(cmd, min_meta_size - pool_metadata_size));
+		pool_metadata_size = min_meta_size;
+	}
+
+	if (!(*pool_metadata_extents =
+	      extents_from_size(cmd, pool_metadata_size, extent_size)))
+		return_0;
+
 	if ((uint64_t) *chunk_size > (uint64_t) pool_data_extents * extent_size) {
 		log_error("Size of %s data volume cannot be smaller than chunk size %s.",
 			  segtype->name, display_size(cmd, *chunk_size));
 		return 0;
 	}
 
-	min_meta_size = _cache_min_metadata_size((uint64_t) pool_data_extents * extent_size, *chunk_size);
-	min_meta_size = dm_round_up(min_meta_size, extent_size);
-
-	if (!pool_metadata_size)
-		pool_metadata_size = min_meta_size;
-
-	if (!update_pool_metadata_min_max(cmd, extent_size,
-					  min_meta_size,
-					  (2 * DEFAULT_CACHE_POOL_MAX_METADATA_SIZE),
-					  &pool_metadata_size,
-					  metadata_lv,
-					  pool_metadata_extents))
-		return_0;
-
 	log_verbose("Preferred pool metadata size %s.",
 		    display_size(cmd, (uint64_t)*pool_metadata_extents * extent_size));
 
@@ -409,7 +425,7 @@ int lv_cache_wait_for_clean(struct logical_volume *cache_lv, int *is_clean)
 	const struct logical_volume *lock_lv = lv_lock_holder(cache_lv);
 	struct lv_segment *cache_seg = first_seg(cache_lv);
 	struct lv_status_cache *status;
-	int cleaner_policy = 0, writeback;
+	int cleaner_policy, writeback;
 	uint64_t dirty_blocks;
 
 	*is_clean = 0;
@@ -417,9 +433,6 @@ int lv_cache_wait_for_clean(struct logical_volume *cache_lv, int *is_clean)
 	//FIXME: use polling to do this...
 	for (;;) {
 		sigint_allow();
-		if (cleaner_policy)
-			/* TODO: Use centralized place */
-			usleep(500000);
 		sigint_restore();
 		if (sigint_caught()) {
 			sigint_clear();
@@ -455,8 +468,13 @@ int lv_cache_wait_for_clean(struct logical_volume *cache_lv, int *is_clean)
 		log_print_unless_silent("Flushing " FMTu64 " blocks for cache %s.",
 					dirty_blocks, display_lvname(cache_lv));
 
-		if (cleaner_policy)
+		if (cleaner_policy) {
+			/* TODO: Use centralized place */
+			sigint_allow();
+			usleep(500000);
+			sigint_restore();
 			continue;
+		}
 
 		if (!(cache_lv->status & LVM_WRITE)) {
 			log_warn("WARNING: Dirty blocks found on read-only cache volume %s.",
@@ -548,8 +566,8 @@ int lv_cache_remove(struct logical_volume *cache_lv)
 		default:
 			/* Otherwise localy activate volume to sync dirty blocks */
 			cache_lv->status |= LV_TEMPORARY;
-			if (!activate_lv_excl_local(cache_lv->vg->cmd, cache_lv) ||
-			    !lv_is_active_locally(cache_lv)) {
+			if (!activate_lv(cache_lv->vg->cmd, cache_lv) ||
+			    !lv_is_active(cache_lv)) {
 				log_error("Failed to active cache locally %s.",
 					  display_lvname(cache_lv));
 				return 0;
@@ -602,13 +620,12 @@ int lv_cache_remove(struct logical_volume *cache_lv)
 	if (!(cache_seg->segtype = get_segtype_from_string(corigin_lv->vg->cmd, SEG_TYPE_NAME_CACHE)))
 		return_0;
 
-	if (!(cache_seg->areas = dm_pool_zalloc(cache_lv->vg->vgmem, sizeof(*cache_seg->areas))))
+	if (!add_lv_segment_areas(cache_seg, 1))
 		return_0;
 
 	if (!set_lv_segment_area_lv(cache_seg, 0, cache_lv, 0, 0))
 		return_0;
 
-	cache_seg->area_count = 1;
 	corigin_lv->le_count = cache_lv->le_count;
 	corigin_lv->size = cache_lv->size;
 	corigin_lv->status |= LV_PENDING_DELETE;
@@ -947,7 +964,7 @@ int wipe_cache_pool(struct logical_volume *cache_pool_lv)
 	}
 
 	cache_pool_lv->status |= LV_TEMPORARY;
-	if (!activate_lv_excl_local(cache_pool_lv->vg->cmd, cache_pool_lv)) {
+	if (!activate_lv(cache_pool_lv->vg->cmd, cache_pool_lv)) {
 		log_error("Aborting. Failed to activate cache pool %s.",
 			  display_lvname(cache_pool_lv));
 		return 0;
